@@ -35,6 +35,7 @@
 #include "misc.h"
 #include "sound.h"
 #include "string.h"
+#include "partyline.h"
 
 extern GConfClient *gconf_client;
 extern GtkWidget *app;
@@ -42,7 +43,7 @@ extern GtkWidget *app;
 /*****************************************************/
 /* connecting dialog - a dialog with a cancel button */
 /*****************************************************/
-static GtkWidget *connectingdialog = 0;
+static GtkWidget *connectingdialog = 0, *connectdialog;
 static GtkWidget *progressbar;
 static gint timeouttag = 0;
 
@@ -56,7 +57,7 @@ void connectingdialog_button (GtkWidget *dialog, gint button)
         gtk_timeout_remove (timeouttag);
         timeouttag = 0;
         if (connectingdialog == 0) return;
-        client_connectcancel ();
+        client_disconnect ();
         gtk_widget_destroy (connectingdialog);
         connectingdialog = 0;
         break;
@@ -82,10 +83,11 @@ void connectingdialog_new (void)
       return;
     }
     connectingdialog = gtk_dialog_new_with_buttons (_("Connect to server"),
-                                                    NULL,
-                                                    0,
+                                                    GTK_WINDOW (connectdialog),
+                                                    GTK_DIALOG_MODAL,
                                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                                     NULL);
+    gtk_window_set_skip_taskbar_hint (GTK_WINDOW (connectingdialog), TRUE);
     progressbar = gtk_progress_bar_new ();
     gtk_widget_show (progressbar);
     gtk_box_pack_start (GTK_BOX (GTK_DIALOG(connectingdialog)->vbox),
@@ -152,11 +154,12 @@ void teamdialog_new (void)
     }
   
     team_dialog = gtk_dialog_new_with_buttons (_("Change team"),
-                                               0,
+                                               GTK_WINDOW (app),
                                                GTK_DIALOG_NO_SEPARATOR,
-                                               GTK_STOCK_CANCEL, GTK_RESPONSE_CLOSE,
+                                               GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                                GTK_STOCK_OK, GTK_RESPONSE_OK,
                                                NULL);
+    gtk_window_set_skip_taskbar_hint (GTK_WINDOW (team_dialog), TRUE);
     gtk_dialog_set_default_response (GTK_DIALOG (team_dialog), GTK_RESPONSE_OK);    
     gtk_window_set_position (GTK_WINDOW (team_dialog), GTK_WIN_POS_MOUSE);
     gtk_window_set_resizable (GTK_WINDOW (team_dialog), FALSE);
@@ -168,6 +171,8 @@ void teamdialog_new (void)
     entry = gnome_entry_new ("Team");
     gtk_entry_set_text (GTK_ENTRY (gnome_entry_gtk_entry (GNOME_ENTRY (entry))),
                         team_utf8);
+    g_object_set (G_OBJECT (gnome_entry_gtk_entry (GNOME_ENTRY (entry))),
+                  "activates_default", TRUE, NULL);
     g_free (team_utf8);
     gtk_box_pack_start_defaults (GTK_BOX (hbox), entry);
     gtk_container_set_border_width (GTK_CONTAINER (hbox), GNOME_PAD_SMALL);
@@ -186,7 +191,7 @@ void teamdialog_new (void)
 /**********************/
 static int connecting;
 static GtkWidget *serveraddressentry, *nicknameentry, *teamnameentry, *spectatorcheck, *passwordentry;
-static GtkWidget *connectdialog, *passwordlabel, *teamnamelabel;
+static GtkWidget *passwordlabel, *teamnamelabel;
 static GtkWidget *originalradio, *tetrifastradio;
 static GSList *gametypegroup;
 static int oldgamemode;
@@ -259,11 +264,12 @@ void connectdialog_button (GtkDialog *dialog, gint button)
         gconf_client_set_string (gconf_client, "/apps/gtetrinet/player/server", server1, NULL);
         gconf_client_set_string (gconf_client, "/apps/gtetrinet/player/nickname", nick, NULL);
         gconf_client_set_string (gconf_client, "/apps/gtetrinet/player/team",
-                                 gtk_entry_get_text (GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(teamnameentry)))), NULL);
+                                 gtk_entry_get_text (GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(teamnameentry)))),
+                                 NULL);
         g_free (team_utf8);
         g_free (nick);
         break;
-    case GTK_RESPONSE_CLOSE:
+    case GTK_RESPONSE_CANCEL:
         gamemode = oldgamemode;
         gtk_widget_destroy (connectdialog);
         break;
@@ -329,7 +335,7 @@ void connectdialog_new (void)
     connectdialog = gtk_dialog_new_with_buttons (_("Connect to server"),
                                                  GTK_WINDOW (app),
                                                  GTK_DIALOG_NO_SEPARATOR,
-                                                 GTK_STOCK_CANCEL, GTK_RESPONSE_CLOSE,
+                                                 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                                  GTK_STOCK_OK, GTK_RESPONSE_OK,
                                                  NULL);
     gtk_dialog_set_default_response (GTK_DIALOG (connectdialog), GTK_RESPONSE_OK);
@@ -346,6 +352,8 @@ void connectdialog_new (void)
     table2 = gtk_table_new (2, 1, FALSE);
 
     serveraddressentry = gnome_entry_new ("Server");
+    g_object_set(G_OBJECT(gnome_entry_gtk_entry(GNOME_ENTRY(serveraddressentry))),
+                 "activates_default", TRUE, NULL);
     gtk_entry_set_text (GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(serveraddressentry))),
                         server);
     gtk_widget_show (serveraddressentry);
@@ -353,9 +361,9 @@ void connectdialog_new (void)
                       0, 1, 0, 1, GTK_FILL | GTK_EXPAND,
                       GTK_FILL | GTK_EXPAND, 0, 0);
     /* game type radio buttons */
-    originalradio = gtk_radio_button_new_with_label (NULL, _("Original"));
+    originalradio = gtk_radio_button_new_with_mnemonic (NULL, _("O_riginal"));
     gametypegroup = gtk_radio_button_get_group (GTK_RADIO_BUTTON(originalradio));
-    tetrifastradio = gtk_radio_button_new_with_label (gametypegroup, _("TetriFast"));
+    tetrifastradio = gtk_radio_button_new_with_mnemonic (gametypegroup, _("Tetri_Fast"));
     switch (gamemode) {
     case ORIGINAL:
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(originalradio), TRUE);
@@ -386,16 +394,19 @@ void connectdialog_new (void)
     /* spectator checkbox + password */
     table2 = gtk_table_new (1, 1, FALSE);
 
-    spectatorcheck = gtk_check_button_new_with_label (_("Connect as a spectator"));
+    spectatorcheck = gtk_check_button_new_with_mnemonic (_("Connect as a _spectator"));
     gtk_widget_show (spectatorcheck);
     gtk_table_attach (GTK_TABLE(table2), spectatorcheck, 0, 2, 0, 1,
                       GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 0, 0);
-    passwordlabel = gtk_label_new (_("Password:"));
+    passwordlabel = gtk_label_new_with_mnemonic (_("_Password:"));
     gtk_widget_show (passwordlabel);
     gtk_table_attach (GTK_TABLE(table2), passwordlabel, 0, 1, 1, 2,
                       GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 0, 0);
     passwordentry = gtk_entry_new ();
+    gtk_label_set_mnemonic_widget (GTK_LABEL (passwordlabel), passwordentry);
     gtk_entry_set_visibility (GTK_ENTRY(passwordentry), FALSE);
+    g_object_set(G_OBJECT(passwordentry),
+                 "activates_default", TRUE, NULL);
     gtk_widget_show (passwordentry);
     gtk_table_attach (GTK_TABLE(table2), passwordentry, 1, 2, 1, 2,
                       GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 0, 0);
@@ -413,11 +424,14 @@ void connectdialog_new (void)
     /* nickname and teamname entries */
     table2 = gtk_table_new (1, 1, FALSE);
 
-    widget = gtk_label_new (_("Nick name:"));
+    widget = gtk_label_new_with_mnemonic (_("_Nick name:"));
     gtk_widget_show (widget);
     gtk_table_attach (GTK_TABLE(table2), widget, 0, 1, 0, 1,
                       GTK_FILL | GTK_EXPAND, 0, 0, 0);
     nicknameentry = gnome_entry_new ("Nickname");
+    gtk_label_set_mnemonic_widget (GTK_LABEL (widget), nicknameentry);
+    g_object_set(G_OBJECT(gnome_entry_gtk_entry(GNOME_ENTRY(nicknameentry))),
+                 "activates_default", TRUE, NULL);
     aux = g_locale_to_utf8 (nick, -1, NULL, NULL, NULL);
     gtk_entry_set_text (GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(nicknameentry))),
                         aux);
@@ -425,11 +439,14 @@ void connectdialog_new (void)
     gtk_widget_show (nicknameentry);
     gtk_table_attach (GTK_TABLE(table2), nicknameentry, 1, 2, 0, 1,
                       GTK_FILL | GTK_EXPAND, 0, 0, 0);
-    teamnamelabel = gtk_label_new (_("Team name:"));
+    teamnamelabel = gtk_label_new_with_mnemonic (_("_Team name:"));
     gtk_widget_show (teamnamelabel);
     gtk_table_attach (GTK_TABLE(table2), teamnamelabel, 0, 1, 1, 2,
                       GTK_FILL | GTK_EXPAND, 0, 0, 0);
     teamnameentry = gnome_entry_new ("Teamname");
+    gtk_label_set_mnemonic_widget (GTK_LABEL (teamnamelabel), teamnameentry);
+    g_object_set(G_OBJECT(gnome_entry_gtk_entry(GNOME_ENTRY(teamnameentry))),
+                 "activates_default", TRUE, NULL);
     aux = g_locale_to_utf8 (team, -1, NULL, NULL, NULL);
     gtk_entry_set_text (GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(teamnameentry))),
                         aux);
@@ -507,6 +524,7 @@ gint key_dialog (char *msg)
 /* the preferences dialog */
 /**************************/
 GtkWidget *themelist, *keyclist;
+GtkWidget *timestampcheck;
 GtkWidget *midientry, *miditable, *midicheck, *soundcheck;
 GtkWidget *namelabel, *authlabel, *desclabel;
 
@@ -541,6 +559,12 @@ void prefdialog_drawkeys (void)
     actions[K_DROP]     = _("Drop piece");
     actions[K_DISCARD]  = _("Discard special");
     actions[K_GAMEMSG]  = _("Send message");
+    actions[K_SPECIAL1] = _("Special to field 1");
+    actions[K_SPECIAL2] = _("Special to field 2");
+    actions[K_SPECIAL3] = _("Special to field 3");
+    actions[K_SPECIAL4] = _("Special to field 4");
+    actions[K_SPECIAL5] = _("Special to field 5");
+    actions[K_SPECIAL6] = _("Special to field 6");
   
     gconf_keys[K_RIGHT]    = g_strdup ("/apps/gtetrinet/keys/right");
     gconf_keys[K_LEFT]     = g_strdup ("/apps/gtetrinet/keys/left");
@@ -550,6 +574,12 @@ void prefdialog_drawkeys (void)
     gconf_keys[K_DROP]     = g_strdup ("/apps/gtetrinet/keys/drop");
     gconf_keys[K_DISCARD]  = g_strdup ("/apps/gtetrinet/keys/discard");
     gconf_keys[K_GAMEMSG]  = g_strdup ("/apps/gtetrinet/keys/message");
+    gconf_keys[K_SPECIAL1] = g_strdup ("/apps/gtetrinet/keys/special1");
+    gconf_keys[K_SPECIAL2] = g_strdup ("/apps/gtetrinet/keys/special2");
+    gconf_keys[K_SPECIAL3] = g_strdup ("/apps/gtetrinet/keys/special3");
+    gconf_keys[K_SPECIAL4] = g_strdup ("/apps/gtetrinet/keys/special4");
+    gconf_keys[K_SPECIAL5] = g_strdup ("/apps/gtetrinet/keys/special5");
+    gconf_keys[K_SPECIAL6] = g_strdup ("/apps/gtetrinet/keys/special6");
 
     for (i = 0; i < K_NUM; i ++) {
         gtk_list_store_append (keys_store, &iter);
@@ -640,6 +670,12 @@ void prefdialog_soundtoggle (GtkWidget *widget)
                            GTK_TOGGLE_BUTTON (widget)->active, NULL);
 }
 
+void prefdialog_channeltoggle (GtkWidget *widget)
+{
+  gconf_client_set_bool (gconf_client, "/apps/gtetrinet/partyline/enable_channel_list",
+			 GTK_TOGGLE_BUTTON (widget)->active, NULL);
+}
+
 void prefdialog_miditoggle (GtkWidget *widget)
 {
     if (GTK_TOGGLE_BUTTON(widget)->active) {
@@ -669,6 +705,12 @@ void prefdialog_restoremidi (void)
                              NULL);
 }
 
+void prefdialog_timestampstoggle (GtkWidget *widget)
+{
+    gconf_client_set_bool (gconf_client, "/apps/gtetrinet/partyline/enable_timestamps",
+                           GTK_TOGGLE_BUTTON (widget)->active, NULL);
+}
+
 void prefdialog_themelistselect (int n)
 {
     char author[1024], desc[1024];
@@ -787,6 +829,7 @@ void prefdialog_new (void)
 {
     GtkWidget *label, *table, *frame, *button, *button1, *widget, *table1, *divider, *notebook;
     GtkWidget *themelist_scroll, *key_scroll;
+    GtkWidget *channel_list_check;
     GtkListStore *theme_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_INT);
     GtkListStore *keys_store = gtk_list_store_new (4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING);
     GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
@@ -804,7 +847,6 @@ void prefdialog_new (void)
                                               GTK_STOCK_HELP, GTK_RESPONSE_HELP,
                                               GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                                               NULL);
-    gtk_dialog_set_default_response (GTK_DIALOG (prefdialog), GTK_RESPONSE_CLOSE);
     notebook = gtk_notebook_new ();
     gtk_window_set_resizable (GTK_WINDOW (prefdialog), FALSE);
 
@@ -878,11 +920,44 @@ void prefdialog_new (void)
     gtk_widget_show (label);
     gtk_notebook_append_page (GTK_NOTEBOOK (notebook), table, label);
 
+    /* partyline */
+    timestampcheck = gtk_check_button_new_with_mnemonic (_("Enable _Timestamps"));
+    gtk_widget_show(timestampcheck);
+    channel_list_check = gtk_check_button_new_with_mnemonic (_("Enable Channel _List"));
+    gtk_widget_show (channel_list_check);
+
+    frame = gtk_vbox_new (FALSE, 0);
+    gtk_box_pack_start (GTK_BOX(frame), timestampcheck, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX(frame), channel_list_check, FALSE, FALSE, 0);
+    gtk_widget_show (frame);
+
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(timestampcheck),
+                                  timestampsenable);
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (channel_list_check),
+				  list_enabled);
+
+    g_signal_connect (G_OBJECT(timestampcheck), "toggled",
+                      G_CALLBACK(prefdialog_timestampstoggle), NULL);
+    g_signal_connect (G_OBJECT (channel_list_check), "toggled",
+		      G_CALLBACK (prefdialog_channeltoggle), NULL);
+
+    table = gtk_table_new (3, 1, FALSE);
+    gtk_container_set_border_width (GTK_CONTAINER(table), GNOME_PAD);
+    gtk_table_set_row_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);
+    gtk_table_set_col_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);
+    gtk_table_attach (GTK_TABLE(table), frame, 0, 1, 0, 1,
+                      GTK_EXPAND | GTK_FILL, 0, 0, 0);
+    gtk_widget_show (table);
+
+    label = gtk_label_new (_("Partyline"));
+    gtk_widget_show (label);
+    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), table, label);
+
     /* keyboard */
     keyclist = GTK_WIDGET (gtk_tree_view_new_with_model (GTK_TREE_MODEL(keys_store)));
     key_scroll = gtk_scrolled_window_new (NULL, NULL);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (key_scroll),
-                                   GTK_POLICY_NEVER,
+                                   GTK_POLICY_AUTOMATIC,
                                    GTK_POLICY_AUTOMATIC);
     gtk_container_add (GTK_CONTAINER(key_scroll), keyclist);
     gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (keyclist), -1, _("Action"), renderer,
@@ -890,7 +965,7 @@ void prefdialog_new (void)
     gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (keyclist), -1, _("Key"), renderer,
                                                  "text", 1, NULL);
 
-    gtk_widget_set_size_request (key_scroll, 180, 200);
+    gtk_widget_set_size_request (key_scroll, 240, 200);
     gtk_widget_show (key_scroll);
 
     label = gtk_label_new (_("Select an action from the list and press Change "
@@ -898,15 +973,16 @@ void prefdialog_new (void)
     gtk_label_set_justify (GTK_LABEL(label), GTK_JUSTIFY_LEFT);
     gtk_label_set_line_wrap (GTK_LABEL(label), TRUE);
     gtk_widget_show (label);
+    gtk_widget_set_size_request (label, 180, 100);
 
-    button = gtk_button_new_with_label (_("Change key..."));
+    button = gtk_button_new_with_mnemonic (_("Change _key..."));
     g_signal_connect (G_OBJECT(button), "clicked",
-                        GTK_SIGNAL_FUNC (prefdialog_changekey), NULL);
+                      GTK_SIGNAL_FUNC (prefdialog_changekey), NULL);
     gtk_widget_show (button);
 
-    button1 = gtk_button_new_with_label (_("Restore defaults"));
+    button1 = gtk_button_new_with_mnemonic (_("_Restore defaults"));
     g_signal_connect (G_OBJECT(button1), "clicked",
-                        GTK_SIGNAL_FUNC (prefdialog_restorekeys), NULL);
+                      GTK_SIGNAL_FUNC (prefdialog_restorekeys), NULL);
     gtk_widget_show (button1);
 
     table = gtk_table_new (2, 2, FALSE);
@@ -930,10 +1006,10 @@ void prefdialog_new (void)
     gtk_notebook_append_page (GTK_NOTEBOOK (notebook), table, label);
 
     /* sound */
-    soundcheck = gtk_check_button_new_with_label (_("Enable Sound"));
+    soundcheck = gtk_check_button_new_with_mnemonic (_("Enable _Sound"));
     gtk_widget_show (soundcheck);
 
-    midicheck = gtk_check_button_new_with_label (_("Enable MIDI"));
+    midicheck = gtk_check_button_new_with_mnemonic (_("Enable _MIDI"));
     gtk_widget_show (midicheck);
 
     frame = gtk_hbox_new (FALSE, 0);
@@ -957,9 +1033,9 @@ void prefdialog_new (void)
     gtk_label_set_line_wrap (GTK_LABEL(label), TRUE);
     gtk_widget_show (label);
 
-    button = gtk_button_new_with_label (_("Restore defaults"));
+    button = gtk_button_new_with_mnemonic (_("_Restore defaults"));
     g_signal_connect (G_OBJECT(button), "clicked",
-                        GTK_SIGNAL_FUNC (prefdialog_restoremidi), NULL);
+                      GTK_SIGNAL_FUNC (prefdialog_restoremidi), NULL);
     gtk_widget_show (button);
 
     miditable = gtk_table_new (4, 2, FALSE);
